﻿using Acme.BookStore.Authors;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Volo.Abp.Guids;
using Volo.Abp;
using Volo.Abp.Domain.Services;

namespace Acme.BookStore.Customers
{
    
    public class CustomerManager : DomainService
    {
        private readonly ICustomerRepository _customerRepository;
        public CustomerManager(ICustomerRepository customerRepository) 
        {
            _customerRepository = customerRepository;
        }

        public async Task<Customer> CreateAsync(
           Customer customer)
        {
            Check.NotNullOrWhiteSpace(customer.firstName, nameof(customer.firstName));
            Check.NotNullOrWhiteSpace(customer.lastName, nameof(customer.lastName));

            var existingAuthor = await _customerRepository.FindByPhoneAsync(customer.phone);
            if (existingAuthor != null)
            {
                throw new CustomerErrorException(customer.phone);
            }
            return new Customer(
                GuidGenerator.Create(),
                customer.firstName,
                    customer.lastName,
                customer.phone,
                customer.address
            );
        }

        public async Task ChangeNameAsync(
         Customer customer,
          string firstName,string lastName)
        {
            Check.NotNull(customer, nameof(customer));
            Check.NotNullOrWhiteSpace(customer.firstName, nameof(customer.firstName));
            Check.NotNullOrWhiteSpace(customer.lastName, nameof(customer.lastName));

            customer.ChangeFirstName(firstName);
            customer.ChangeLastName(lastName);
        }

    }
}
