﻿using Acme.BookStore.Customers;
using Acme.BookStore.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore;
using Polly;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Volo.Abp;
using Volo.Abp.Data;
using Volo.Abp.Domain.Repositories.EntityFrameworkCore;
using Volo.Abp.EntityFrameworkCore;
using static Microsoft.EntityFrameworkCore.DbLoggerCategory;

namespace Acme.BookStore.BookIssued
{
    public class EfCoreBookIssueRepository : EfCoreRepository<BookStoreDbContext, BookIssue, Guid>,
        IBookIssueRepository
    {
        private readonly IDataFilter _dataFilter;

        public EfCoreBookIssueRepository(
            IDbContextProvider<BookStoreDbContext> dbContextProvider, IDataFilter dataFilter
)
            : base(dbContextProvider)
        {
            _dataFilter = dataFilter;

        }
        public async Task<BookIssueList> FindBookIssueCustomer(Guid customerId, Guid bookId)
        {
            //var dbSet = await GetDbSetAsync();

            var dbContext = await GetDbContextAsync();

            List<BookIssueList> item = (
            from bi in dbContext.bookIssues
            join cu in dbContext.Customers on bi.customerId equals cu.Id
            join bk in dbContext.Books on bi.bookId equals bk.Id
            where (bi.customerId == customerId && bi.bookId == bookId && bi.IsDeleted == false)
            select new BookIssueList
            {
                bookIssueId = bi.Id,
                bookName = bk.Name,
                customerName = cu.firstName + " " + cu.lastName,
                issueDate = bi.CreationTime
            }).ToList();

            return item == null? null : item.FirstOrDefault();
             
        }

        //public async Task<int> NewBookIssueId()
        //{
        //    using (_dataFilter.Disable<ISoftDelete>())
        //    {
        //        var dbSet = await GetDbSetAsync();
        //        var values = await dbSet.MaxAsync(u => (Guid?)u.Id);
        //        return values == null ? 0 + 1 : (int)values + 1;

        //    }

        //}

        public async Task<List<BookIssueList>> BookIssueList()
        {
            var dbContext = await GetDbContextAsync();

            List<BookIssueList> item = (
            from bi in dbContext.bookIssues
            join cu in dbContext.Customers on bi.customerId equals cu.Id
            join bk in dbContext.Books on bi.bookId equals bk.Id
            select new BookIssueList
            {
                bookIssueId = bi.Id,
                bookName = bk.Name,
                customerName = cu.firstName + " " + cu.lastName,
                issueDate = bi.CreationTime
            }).ToList();

            return item;
        }
    }
}
